Package: reticulate
Type: Package
Title: Interface to 'Python'
Version: 1.47.0
Authors@R: c(
  person("Tomasz", "Kalinowski", role = c("aut", "cre"),
         email = "tomasz@posit.co"),
  person("Kevin", "Ushey", role = c("aut"),
         email = "kevin@posit.co"),
  person("JJ", "Allaire", role = c("aut"), email = "jj@posit.co"),
  person("RStudio", role = c("cph", "fnd")),
  person("Yuan", "Tang", role = c("aut", "cph"),
         email = "terrytangyuan@gmail.com",
         comment = c(ORCID = "0000-0001-5243-233X")),
  person("Dirk", "Eddelbuettel", role = c("ctb", "cph"),
         email = "edd@debian.org"),
  person("Bryan", "Lewis", role = c("ctb", "cph"),
         email = "blewis@illposed.net"),
  person("Sigrid", "Keydana", role = c("ctb"),
         email = "sigrid@posit.co"),
  person("Ryan", "Hafen", role = c("ctb", "cph"),
         email = "rhafen@gmail.com"),
  person("Marcus", "Geelnard", role = c("ctb", "cph"),
         comment = "TinyThread library, http://tinythreadpp.bitsnbites.eu/")
  )
Description: Interface to 'Python' modules, classes, and functions. When calling
    into 'Python', R data types are automatically converted to their equivalent 'Python'
    types. When values are returned from 'Python' to R they are converted back to R
    types. Compatible with all versions of 'Python' >= 2.7.
License: Apache License 2.0
URL: https://rstudio.github.io/reticulate/,
        https://github.com/rstudio/reticulate
BugReports: https://github.com/rstudio/reticulate/issues
SystemRequirements: Python (>= 2.7.0)
Encoding: UTF-8
Depends: R (>= 3.5)
Imports: Matrix, Rcpp (>= 1.0.7), RcppTOML, graphics, here, jsonlite,
        methods, png, rappdirs, utils, rlang, withr
Suggests: callr, knitr, glue, cli, rmarkdown, pillar, testthat
LinkingTo: Rcpp
VignetteBuilder: knitr
Config/build/compilation-database: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-09-02 16:31:29 UTC; tomasz
Author: Tomasz Kalinowski [aut, cre],
  Kevin Ushey [aut],
  JJ Allaire [aut],
  RStudio [cph, fnd],
  Yuan Tang [aut, cph] (ORCID: <https://orcid.org/0000-0001-5243-233X>),
  Dirk Eddelbuettel [ctb, cph],
  Bryan Lewis [ctb, cph],
  Sigrid Keydana [ctb],
  Ryan Hafen [ctb, cph],
  Marcus Geelnard [ctb, cph] (TinyThread library,
    http://tinythreadpp.bitsnbites.eu/)
Maintainer: Tomasz Kalinowski <tomasz@posit.co>
Repository: CRAN
Date/Publication: 2026-09-03 05:20:02 UTC
