% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/help.R
\name{py_help}
\alias{py_help}
\title{Documentation for Python Objects}
\usage{
py_help(object)
}
\arguments{
\item{object}{Object to print documentation for}
}
\description{
Documentation for Python Objects
}
