% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/config.R
\name{py_discover_config}
\alias{py_discover_config}
\title{Discover the version of Python to use with reticulate.}
\usage{
py_discover_config(required_module = NULL, use_environment = NULL)
}
\arguments{
\item{required_module}{A optional module name that will be used to select the
Python environment used.}

\item{use_environment}{An optional virtual/conda environment name to prefer
in the search.}
}
\value{
Python configuration object.
}
\description{
This function enables callers to check which versions of Python will be
discovered on a system as well as which one will be chosen for use with
reticulate.
}
\details{
The order of discovery is documented in \code{vignette("versions")}, also available online
\href{https://rstudio.github.io/reticulate/articles/versions.html#order-of-discovery}{here}
}
