% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/python.R
\name{py_ellipsis}
\alias{py_ellipsis}
\title{The builtin constant Ellipsis}
\usage{
py_ellipsis()
}
\description{
The builtin constant Ellipsis
}
