% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/python.R
\name{py_none}
\alias{py_none}
\title{The Python None object}
\usage{
py_none()
}
\description{
Get a reference to the Python \code{None} object.
}
