% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/repl.R
\name{ipython}
\alias{ipython}
\title{IPython console}
\usage{
ipython()
}
\description{
Launch IPython console app.
}
\details{
See https://ipython.readthedocs.io/ for features.
}
\keyword{internal}
