% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{reticulate}
\alias{reticulate}
\title{R Interface to Python}
\description{
R interface to Python modules, classes, and functions. When calling into
Python R data types are automatically converted to their equivalent Python
types. When values are returned from Python to R they are converted back to R
types. The reticulate package is compatible with all versions of Python >= 2.7.
Integration with NumPy requires NumPy version 1.6 or higher.
}
